import os
import subprocess
from setuptools import setup, find_packages
from pathlib import Path

def get_package_data():
    """获取assets目录中要包含为包数据的所有文件"""
    assets_path = Path(__file__).parent
    
    # 获取所有git忽略的文件
    result = subprocess.run(
        ["git", "ls-files", "--ignored", "--exclude-standard", "--others"],
        cwd=assets_path,
        text=True,
        capture_output=True
    )
    ignored_files = result.stdout.splitlines()
    
    package_data_files = []
    for file in ignored_files:
        # 排除路径中包含__MACOSX的文件
        if "__MACOSX" in file:
            continue
        file_path = assets_path / file
        if file_path.exists() and file_path.is_file():
            package_data_files.append(file)
    return package_data_files

setup(
    name="robocasa-assets",
    description="Static assets for RoboCasa",
    author="Soroush Nasiriany, Abhiram Maddukuri, Lance Zhang, Adeet Parikh, Aaron Lo, Abhishek Joshi, Ajay Mandlekar, Yuke Zhu",
    url="https://github.com/robocasa/robocasa",
    author_email="soroush@cs.utexas.edu",
    python_requires=">=3",
    install_requires=[],
    
    packages=["robocasa.models.assets"],
    package_dir={"robocasa.models.assets": "."},
    package_data={
        "robocasa.models.assets": get_package_data(),
    },
    # 确保setuptools包含包数据
    include_package_data=True,
)
